/*
 * Copyright (c) 2002-2004 David Keiichi Watanabe
 * davew@xlife.org
 *
 * Modified by (c) 2004-2005 heavy_baby
 * heavy_baby@users.sourceforge.jp
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jp.sourceforge.cabos;

import com.limegroup.gnutella.*;
import com.limegroup.gnutella.browser.*;
import com.limegroup.gnutella.downloader.*;
/* classic
import com.limegroup.gnutella.gui.*;
*/
import com.limegroup.gnutella.search.*;
import com.limegroup.gnutella.security.*;
import com.limegroup.gnutella.settings.*;
import com.limegroup.gnutella.util.*;
import java.io.*;
/* classic
import java.util.*;
*/
import com.sun.java.util.collections.*;


public class AqEventHandler implements ActivityCallback, ErrorCallback, MessageCallback
{
    
    /* Instances */
    
    protected volatile static Map stateCache        = new Hashtable();
    protected volatile static Map hosts             = new Hashtable();
    protected volatile static Map queriesToIndices  = new HashMap();
    protected volatile static Map indicesToQueries  = new HashMap();
    protected volatile static Map queryGUIDs        = new HashMap();
    protected volatile static Map responses         = new HashMap();
    protected volatile static Map alts              = new HashMap();
    protected volatile static int currentQueryIndex = -1;
    protected volatile static int currentLocalIndex = -1;
   
    /* Connections */
    
    public void addressStateChanged() {
    }
    
    public void connectionInitializing(Connection c) {
    }
    
    public void connectionInitialized(Connection c) {
        if (c instanceof ManagedConnection)
            AqEvent.signalEvent(AqEvent.kLWEventConnectionInitialized, c);
    }
    
    public void connectionClosed(Connection c) {
        if (c instanceof ManagedConnection)
            AqEvent.signalEvent(AqEvent.kLWEventConnectionClosed, c);
    }
    
    /* Queries */
    
    public void handleQueryResult(RemoteFileDesc rfd, HostData data, Set locs) {
        synchronized (queriesToIndices) {
            if (queriesToIndices.containsKey(new GUID(data.getMessageGUID())))
                AqEvent.signalEvent(AqEvent.kLWEventQueryResult, rfd, data, locs);
        }
    }
    
    public void handleQueryString(String query) {
    }
    
    /* Uploads */
    
    public void addUpload(Uploader u) {
        AqEvent.signalEvent(AqEvent.kLWEventAddUpload, u);
    }
    
    public void removeUpload(Uploader u) {
        AqEvent.signalEvent(AqEvent.kLWEventUpdateUploadStats, u);
        AqEvent.signalEvent(AqEvent.kLWEventRemoveUpload, u);
    }
    
    /* Browse */
    
    public void browseHostFailed(GUID guid) {
    }
    
    /* Sharing */
    
/* classic
	public void fileManagerLoading() {
        if (CommonUtils.isJava14OrLater() && DaapSettings.DAAP_ENABLED.getValue())
            DaapManager.instance().fileManagerLoading();
    }
*/

    public void fileManagerLoaded() {
/* classic
        if (CommonUtils.isJava14OrLater() && DaapSettings.DAAP_ENABLED.getValue())
            DaapManager.instance().fileManagerLoaded();
*/
    }

/* classic */
    public void addSharedDirectory(final File directory, final File parent) {
    }

    public void addSharedFile(final FileDesc file, final File parent) {
        AqEvent.signalEvent(AqEvent.kLWEventAddSharedFile, file);
    }

    public boolean warnAboutSharingSensitiveDirectory(final File dir) {
        return false;
    }
    
/* classic
    public void handleFileManagerEvent(FileManagerEvent evt) {
        if (CommonUtils.isJava14OrLater() && 
            DaapSettings.DAAP_ENABLED.getValue() && 
            DaapManager.instance().isEnabled())
            DaapManager.instance().handleFileManagerEvent(evt);
        if (evt.isAddEvent()) {
            FileDesc[] files = evt.getFileDescs();
            for (int i = 0; i < files.length; i++)
                AqEvent.signalEvent(AqEvent.kLWEventAddSharedFile, files[i]);
        }
    }
*/
    
    public void handleSharedFileUpdate(File file) {
    }

/* classic */
	public void clearSharedFiles() {
    }
   
    public void setAnnotateEnabled(boolean enabled) {
    }
    
    /* Misc */
    
/* classic */
    public User getUserAuthenticationInfo(String host) {
        return null;
    }
    
    public void uploadsComplete() {
    }
    
    public void restoreApplication() {
    }
    
    public boolean isQueryAlive(GUID guid) {
        synchronized (queriesToIndices) {
            return queriesToIndices.containsKey(guid);
        }
    }
    
    public void componentLoading(String component) {
        System.err.println(component);
    }
    
	public boolean handleMagnets(final MagnetOptions[] magnets) {
        return false;
    }

	public void acceptedIncomingChanged(boolean status) {
    }

    /* DownloadCallback */
    
    public void addDownload(Downloader d) {
        if (d instanceof ManagedDownloader)
            AqEvent.signalEvent(AqEvent.kLWEventAddDownload, d);
    }
    
    public void removeDownload(Downloader d) {
        if (d instanceof ManagedDownloader) {
            AqEvent.signalEvent(AqEvent.kLWEventUpdateDownloadStats, d);
            AqEvent.signalEvent(AqEvent.kLWEventRemoveDownload, d);
        }
    }

    public void downloadsComplete() {
    }

	public void showDownloads() {
    }

    public void promptAboutCorruptDownload(Downloader dloader) {
        if (dloader instanceof ManagedDownloader)
            dloader.discardCorruptDownload(false);
    }
    
    public String getHostValue(String key) {
        return null;
    }

    /* ErrorCallback */
    
    public void error(Throwable t) {
        t.printStackTrace();
    }
    
    public void error(Throwable t, String msg) {
        System.err.println(msg);
        t.printStackTrace();
    }
    
    /* MessageCallback */

    public void showError(String messageKey) {
        System.err.println(messageKey);
    }
    
    public void showError(String messageKey, BooleanSetting ignore) {
        System.err.println(messageKey + ": " + ignore);
    }
    
    public void showError(String messageKey, String message) {
        System.err.println(messageKey + ": " + message);
    }
    
    public void showError(String messageKey, String message, BooleanSetting ignore) {
        System.err.println(messageKey + ": " + message + ": " + ignore);
    }
    
    public void showMessage(String messageKey) {
        System.err.println(messageKey);
    }
    
    public void showMessage(String messageKey, BooleanSetting ignore) {
        System.err.println(messageKey + ": " + ignore);
    }

}